@@ -1,4 +1,4 @@
-
+from distutils.version import LooseVersion
 import pickle
 import unittest
 import pytest
@@ -12,7 +12,7 @@ from sklearn.utils.testing import assert_array_almost_equal
 from sklearn.utils.testing import assert_greater
 from sklearn.utils.testing import assert_less
 from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import assert_false, assert_true
+from sklearn.utils.testing import assert_false
 from sklearn.utils.testing import assert_equal
 from sklearn.utils.testing import assert_raises_regexp
 from sklearn.utils.testing import assert_warns
@@ -21,13 +21,22 @@ from sklearn.utils.testing import assert_no_warnings
 from sklearn.utils.testing import ignore_warnings
 
 from sklearn import linear_model, datasets, metrics
-from sklearn.base import clone
+from sklearn.base import clone, is_classifier
 from sklearn.linear_model import SGDClassifier, SGDRegressor
 from sklearn.preprocessing import LabelEncoder, scale, MinMaxScaler
 from sklearn.preprocessing import StandardScaler
 from sklearn.exceptions import ConvergenceWarning
-from sklearn.model_selection import train_test_split
+from sklearn.model_selection import StratifiedShuffleSplit, ShuffleSplit
 from sklearn.linear_model import sgd_fast
+from sklearn.model_selection import RandomizedSearchCV
+
+from sklearn.utils import _joblib
+from sklearn.utils._joblib import parallel_backend
+
+
+# 0.23. warning about tol not having its correct default value.
+pytestmark = pytest.mark.filterwarnings(
+    "ignore:max_iter and tol parameters have been")
 
 
 class SparseSGDClassifier(SGDClassifier):
@@ -217,10 +226,10 @@ class CommonTest(object):
         clf = self.factory(average=True, eta0=.01)
         clf.fit(X, Y)
 
-        assert_true(hasattr(clf, 'average_coef_'))
-        assert_true(hasattr(clf, 'average_intercept_'))
-        assert_true(hasattr(clf, 'standard_intercept_'))
-        assert_true(hasattr(clf, 'standard_coef_'))
+        assert hasattr(clf, 'average_coef_')
+        assert hasattr(clf, 'average_intercept_')
+        assert hasattr(clf, 'standard_intercept_')
+        assert hasattr(clf, 'standard_coef_')
 
         clf = self.factory()
         clf.fit(X, Y)
@@ -278,14 +287,13 @@ class CommonTest(object):
                       alpha=0, learning_rate="optimal")
 
     def test_early_stopping(self):
+        X = iris.data[iris.target > 0]
+        Y = iris.target[iris.target > 0]
         for early_stopping in [True, False]:
             max_iter = 1000
             clf = self.factory(early_stopping=early_stopping, tol=1e-3,
                                max_iter=max_iter).fit(X, Y)
             assert clf.n_iter_ < max_iter
-            assert not hasattr(clf, '_X_val')
-            assert not hasattr(clf, '_y_val')
-            assert not hasattr(clf, '_sample_weight_val')
 
     def test_adaptive_longer_than_constant(self):
         clf1 = self.factory(learning_rate="adaptive", eta0=0.01, tol=1e-3,
@@ -299,28 +307,37 @@ class CommonTest(object):
     def test_validation_set_not_used_for_training(self):
         X, Y = iris.data, iris.target
         validation_fraction = 0.4
-        random_state = 42
+        seed = 42
         shuffle = False
-        clf1 = self.factory(early_stopping=True, random_state=random_state,
+        max_iter = 10
+        clf1 = self.factory(early_stopping=True,
+                            random_state=np.random.RandomState(seed),
                             validation_fraction=validation_fraction,
                             learning_rate='constant', eta0=0.01,
-                            tol=None, max_iter=1000, shuffle=shuffle)
+                            tol=None, max_iter=max_iter, shuffle=shuffle)
         clf1.fit(X, Y)
+        assert clf1.n_iter_ == max_iter
 
-        idx_train, idx_val = train_test_split(
-            np.arange(X.shape[0]), test_size=validation_fraction,
-            random_state=random_state)
         clf2 = self.factory(early_stopping=False,
-                            random_state=random_state,
+                            random_state=np.random.RandomState(seed),
                             learning_rate='constant', eta0=0.01,
-                            tol=None, max_iter=1000, shuffle=shuffle)
+                            tol=None, max_iter=max_iter, shuffle=shuffle)
+
+        if is_classifier(clf2):
+            cv = StratifiedShuffleSplit(test_size=validation_fraction,
+                                        random_state=seed)
+        else:
+            cv = ShuffleSplit(test_size=validation_fraction,
+                              random_state=seed)
+        idx_train, idx_val = next(cv.split(X, Y))
         idx_train = np.sort(idx_train)  # remove shuffling
-        clf2.fit(X[idx_train], np.array(Y)[idx_train])
+        clf2.fit(X[idx_train], Y[idx_train])
+        assert clf2.n_iter_ == max_iter
 
         assert_array_equal(clf1.coef_, clf2.coef_)
 
-    @ignore_warnings(category=ConvergenceWarning)
     def test_n_iter_no_change(self):
+        X, Y = iris.data, iris.target
         # test that n_iter_ increases monotonically with n_iter_no_change
         for early_stopping in [True, False]:
             n_iter_list = [self.factory(early_stopping=early_stopping,
@@ -521,7 +538,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf.fit(X2, Y2, coef_init=np.zeros((3, 2)),
                 intercept_init=np.zeros(3))
         assert_equal(clf.coef_.shape, (3, 2))
-        assert_true(clf.intercept_.shape, (3,))
+        assert clf.intercept_.shape, (3,)
         pred = clf.predict(T2)
         assert_array_equal(pred, true_result2)
 
@@ -569,10 +586,10 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
                 assert not hasattr(clf, 'predict_proba')
                 assert not hasattr(clf, 'predict_log_proba')
                 with pytest.raises(AttributeError,
-                                   message=message):
+                                   match=message):
                     clf.predict_proba
                 with pytest.raises(AttributeError,
-                                   message=message):
+                                   match=message):
                     clf.predict_log_proba
 
     def test_sgd_proba(self):
@@ -592,14 +609,14 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
             clf = self.factory(loss=loss, alpha=0.01, max_iter=10)
             clf.fit(X, Y)
             p = clf.predict_proba([[3, 2]])
-            assert_true(p[0, 1] > 0.5)
+            assert p[0, 1] > 0.5
             p = clf.predict_proba([[-1, -1]])
-            assert_true(p[0, 1] < 0.5)
+            assert p[0, 1] < 0.5
 
             p = clf.predict_log_proba([[3, 2]])
-            assert_true(p[0, 1] > p[0, 0])
+            assert p[0, 1] > p[0, 0]
             p = clf.predict_log_proba([[-1, -1]])
-            assert_true(p[0, 1] < p[0, 0])
+            assert p[0, 1] < p[0, 0]
 
         # log loss multiclass probability estimates
         clf = self.factory(loss="log", alpha=0.01, max_iter=10).fit(X2, Y2)
@@ -608,7 +625,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         p = clf.predict_proba([[.1, -.1], [.3, .2]])
         assert_array_equal(np.argmax(p, axis=1), np.argmax(d, axis=1))
         assert_almost_equal(p[0].sum(), 1)
-        assert_true(np.all(p[0] >= 0))
+        assert np.all(p[0] >= 0)
 
         p = clf.predict_proba([[-1, -1]])
         d = clf.decision_function([[-1, -1]])
@@ -662,13 +679,13 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
 
         # test sparsify with dense inputs
         clf.sparsify()
-        assert_true(sp.issparse(clf.coef_))
+        assert sp.issparse(clf.coef_)
         pred = clf.predict(X)
         assert_array_equal(pred, Y)
 
         # pickle and unpickle with sparse coef_
         clf = pickle.loads(pickle.dumps(clf))
-        assert_true(sp.issparse(clf.coef_))
+        assert sp.issparse(clf.coef_)
         pred = clf.predict(X)
         assert_array_equal(pred, Y)
 
@@ -824,7 +841,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf.partial_fit(X[third:], Y[third:])
         id2 = id(clf.coef_.data)
         # check that coef_ haven't been re-allocated
-        assert_true(id1, id2)
+        assert id1, id2
 
         y_pred = clf.predict(T)
         assert_array_equal(y_pred, true_result)
@@ -843,7 +860,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf.partial_fit(X2[third:], Y2[third:])
         id2 = id(clf.coef_.data)
         # check that coef_ haven't been re-allocated
-        assert_true(id1, id2)
+        assert id1, id2
 
     def test_partial_fit_multiclass_average(self):
         third = X2.shape[0] // 3
@@ -923,7 +940,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         # Test multiple calls of fit w/ different shaped inputs.
         clf = self.factory(alpha=0.01, shuffle=False)
         clf.fit(X, Y)
-        assert_true(hasattr(clf, "coef_"))
+        assert hasattr(clf, "coef_")
 
         # Non-regression test: try fitting with a different label set.
         y = [["ham", "spam"][i] for i in LabelEncoder().fit_transform(Y)]
@@ -1076,7 +1093,7 @@ class DenseSGDRegressorTestCase(unittest.TestCase, CommonTest):
                            fit_intercept=False)
         clf.fit(X, y)
         score = clf.score(X, y)
-        assert_true(score > 0.99)
+        assert score > 0.99
 
         # simple linear function with noise
         y = 0.5 * X.ravel() + rng.randn(n_samples, 1).ravel()
@@ -1086,7 +1103,7 @@ class DenseSGDRegressorTestCase(unittest.TestCase, CommonTest):
                            fit_intercept=False)
         clf.fit(X, y)
         score = clf.score(X, y)
-        assert_true(score > 0.5)
+        assert score > 0.5
 
     def test_sgd_huber_fit(self):
         xmin, xmax = -5, 5
@@ -1153,7 +1170,7 @@ class DenseSGDRegressorTestCase(unittest.TestCase, CommonTest):
         clf.partial_fit(X[third:], Y[third:])
         id2 = id(clf.coef_.data)
         # check that coef_ haven't been re-allocated
-        assert_true(id1, id2)
+        assert id1, id2
 
     def _test_partial_fit_equal_fit(self, lr):
         clf = self.factory(alpha=0.01, max_iter=2, eta0=0.01,
@@ -1227,13 +1244,13 @@ def test_underflow_or_overlow():
 
         X = rng.normal(size=(n_samples, n_features))
         X[:, :2] *= 1e300
-        assert_true(np.isfinite(X).all())
+        assert np.isfinite(X).all()
 
         # Use MinMaxScaler to scale the data without introducing a numerical
         # instability (computing the standard deviation naively is not possible
         # on this data)
         X_scaled = MinMaxScaler().fit_transform(X)
-        assert_true(np.isfinite(X_scaled).all())
+        assert np.isfinite(X_scaled).all()
 
         # Define a ground truth on the scaled data
         ground_truth = rng.normal(size=n_features)
@@ -1244,7 +1261,7 @@ def test_underflow_or_overlow():
 
         # smoke test: model is stable on scaled data
         model.fit(X_scaled, y)
-        assert_true(np.isfinite(model.coef_).all())
+        assert np.isfinite(model.coef_).all()
 
         # model is numerically unstable on unscaled data
         msg_regxp = (r"Floating-point under-/overflow occurred at epoch #.*"
@@ -1261,7 +1278,7 @@ def test_numerical_stability_large_gradient():
                           eta0=0.001, random_state=0, tol=None)
     with np.errstate(all='raise'):
         model.fit(iris.data, iris.target)
-    assert_true(np.isfinite(model.coef_).all())
+    assert np.isfinite(model.coef_).all()
 
 
 @pytest.mark.parametrize('penalty', ['l2', 'l1', 'elasticnet'])
@@ -1321,8 +1338,11 @@ def test_future_and_deprecation_warnings():
     msg_deprecation = "n_iter parameter is deprecated"
     assert_warns_message(DeprecationWarning, msg_deprecation, init, 6, 0, 5)
 
-    # When n_iter=None, and at least one of tol and max_iter is specified
-    assert_no_warnings(init, 100, None, None)
+    # When n_iter=None and max_iter is specified but tol=None
+    msg_changed = "If max_iter is set but tol is left unset"
+    assert_warns_message(FutureWarning, msg_changed, init, 100, None, None)
+
+    # When n_iter=None and tol is specified
     assert_no_warnings(init, None, 1e-3, None)
     assert_no_warnings(init, 100, 1e-3, None)
 
@@ -1471,3 +1491,79 @@ def test_gradient_squared_epsilon_insensitive():
         (2.0, 2.2, -0.2), (-2.0, 1.0, -5.8)
     ]
     _test_gradient_common(loss, cases)
+
+
+def test_multi_thread_multi_class_and_early_stopping():
+    # This is a non-regression test for a bad interaction between
+    # early stopping internal attribute and thread-based parallelism.
+    clf = SGDClassifier(alpha=1e-3, tol=1e-3, max_iter=1000,
+                        early_stopping=True, n_iter_no_change=100,
+                        random_state=0, n_jobs=2)
+    clf.fit(iris.data, iris.target)
+    assert clf.n_iter_ > clf.n_iter_no_change
+    assert clf.n_iter_ < clf.n_iter_no_change + 20
+    assert clf.score(iris.data, iris.target) > 0.8
+
+
+def test_multi_core_gridsearch_and_early_stopping():
+    # This is a non-regression test for a bad interaction between
+    # early stopping internal attribute and process-based multi-core
+    # parallelism.
+    param_grid = {
+        'alpha': np.logspace(-4, 4, 9),
+        'n_iter_no_change': [5, 10, 50],
+    }
+    clf = SGDClassifier(tol=1e-3, max_iter=1000, early_stopping=True,
+                        random_state=0)
+    search = RandomizedSearchCV(clf, param_grid, n_iter=10, cv=5, n_jobs=2,
+                                random_state=0)
+    search.fit(iris.data, iris.target)
+    assert search.best_score_ > 0.8
+
+
+@pytest.mark.skipif(
+        not hasattr(sp, "random"),
+        reason="this test uses scipy.random, that was introduced in version  "
+        "0.17. This skip condition can be dropped as soon as we drop support "
+        "for scipy versions older than 0.17")
+@pytest.mark.parametrize("backend",
+                         ["loky", "multiprocessing", "threading"])
+def test_SGDClassifier_fit_for_all_backends(backend):
+    # This is a non-regression smoke test. In the multi-class case,
+    # SGDClassifier.fit fits each class in a one-versus-all fashion using
+    # joblib.Parallel.  However, each OvA step updates the coef_ attribute of
+    # the estimator in-place. Internally, SGDClassifier calls Parallel using
+    # require='sharedmem'. This test makes sure SGDClassifier.fit works
+    # consistently even when the user asks for a backend that does not provide
+    # sharedmem semantics.
+
+    # We further test a case where memmapping would have been used if
+    # SGDClassifier.fit was called from a loky or multiprocessing backend. In
+    # this specific case, in-place modification of clf.coef_ would have caused
+    # a segmentation fault when trying to write in a readonly memory mapped
+    # buffer.
+
+    if _joblib.__version__ < LooseVersion('0.12') and backend == 'loky':
+        pytest.skip('loky backend does not exist in joblib <0.12')
+
+    random_state = np.random.RandomState(42)
+
+    # Create a classification problem with 50000 features and 20 classes. Using
+    # loky or multiprocessing this make the clf.coef_ exceed the threshold
+    # above which memmaping is used in joblib and loky (1MB as of 2018/11/1).
+    X = sp.random(1000, 50000, density=0.01, format='csr',
+                  random_state=random_state)
+    y = random_state.choice(20, 1000)
+
+    # Begin by fitting a SGD classifier sequentially
+    clf_sequential = SGDClassifier(tol=1e-3, max_iter=1000, n_jobs=1,
+                                   random_state=42)
+    clf_sequential.fit(X, y)
+
+    # Fit a SGDClassifier using the specified backend, and make sure the
+    # coefficients are equal to those obtained using a sequential fit
+    clf_parallel = SGDClassifier(tol=1e-3, max_iter=1000, n_jobs=4,
+                                 random_state=42)
+    with parallel_backend(backend=backend):
+        clf_parallel.fit(X, y)
+    assert_array_almost_equal(clf_sequential.coef_, clf_parallel.coef_)
