@@ -33,7 +33,7 @@ from .utils.multiclass import _check_partial_fit_first_call
 from .utils.validation import check_is_fitted
 from .externals import six
 
-__all__ = ['BernoulliNB', 'GaussianNB', 'MultinomialNB']
+__all__ = ['BernoulliNB', 'GaussianNB', 'MultinomialNB', 'ComplementNB']
 
 
 class BaseNB(six.with_metaclass(ABCMeta, BaseEstimator, ClassifierMixin)):
@@ -122,6 +122,10 @@ class GaussianNB(BaseNB):
         Prior probabilities of the classes. If specified the priors are not
         adjusted according to the data.
 
+    var_smoothing : float, optional (default=1e-9)
+        Portion of the largest variance of all features that is added to
+        variances for calculation stability.
+
     Attributes
     ----------
     class_prior_ : array, shape (n_classes,)
@@ -136,6 +140,9 @@ class GaussianNB(BaseNB):
     sigma_ : array, shape (n_classes, n_features)
         variance of each feature per class
 
+    epsilon_ : float
+        absolute additive value to variances
+
     Examples
     --------
     >>> import numpy as np
@@ -144,18 +151,19 @@ class GaussianNB(BaseNB):
     >>> from sklearn.naive_bayes import GaussianNB
     >>> clf = GaussianNB()
     >>> clf.fit(X, Y)
-    GaussianNB(priors=None)
+    GaussianNB(priors=None, var_smoothing=1e-09)
     >>> print(clf.predict([[-0.8, -1]]))
     [1]
     >>> clf_pf = GaussianNB()
     >>> clf_pf.partial_fit(X, Y, np.unique(Y))
-    GaussianNB(priors=None)
+    GaussianNB(priors=None, var_smoothing=1e-09)
     >>> print(clf_pf.predict([[-0.8, -1]]))
     [1]
     """
 
-    def __init__(self, priors=None):
+    def __init__(self, priors=None, var_smoothing=1e-9):
         self.priors = priors
+        self.var_smoothing = var_smoothing
 
     def fit(self, X, y, sample_weight=None):
         """Fit Gaussian Naive Bayes according to X, y
@@ -178,7 +186,6 @@ class GaussianNB(BaseNB):
         Returns
         -------
         self : object
-            Returns self.
         """
         X, y = check_X_y(X, y)
         return self._partial_fit(X, y, np.unique(y), _refit=True,
@@ -297,7 +304,6 @@ class GaussianNB(BaseNB):
         Returns
         -------
         self : object
-            Returns self.
         """
         return self._partial_fit(X, y, classes, _refit=False,
                                  sample_weight=sample_weight)
@@ -321,7 +327,7 @@ class GaussianNB(BaseNB):
             Must be provided at the first call to partial_fit, can be omitted
             in subsequent calls.
 
-        _refit: bool, optional (default=False)
+        _refit : bool, optional (default=False)
             If true, act as though this were the first time we called
             _partial_fit (ie, throw away any past fitting and start over).
 
@@ -331,7 +337,6 @@ class GaussianNB(BaseNB):
         Returns
         -------
         self : object
-            Returns self.
         """
         X, y = check_X_y(X, y)
         if sample_weight is not None:
@@ -342,7 +347,7 @@ class GaussianNB(BaseNB):
         # will cause numerical errors. To address this, we artificially
         # boost the variance by epsilon, a small fraction of the standard
         # deviation of the largest dimension.
-        epsilon = 1e-9 * np.var(X, axis=0).max()
+        self.epsilon_ = self.var_smoothing * np.var(X, axis=0).max()
 
         if _refit:
             self.classes_ = None
@@ -358,7 +363,6 @@ class GaussianNB(BaseNB):
             self.class_count_ = np.zeros(n_classes, dtype=np.float64)
 
             # Initialise the class prior
-            n_classes = len(self.classes_)
             # Take into account the priors
             if self.priors is not None:
                 priors = np.asarray(self.priors)
@@ -367,7 +371,7 @@ class GaussianNB(BaseNB):
                     raise ValueError('Number of priors must match number of'
                                      ' classes.')
                 # Check that the sum is 1
-                if priors.sum() != 1.0:
+                if not np.isclose(priors.sum(), 1.0):
                     raise ValueError('The sum of the priors should be 1.')
                 # Check that the prior are non-negative
                 if (priors < 0).any():
@@ -382,7 +386,7 @@ class GaussianNB(BaseNB):
                 msg = "Number of features %d does not match previous data %d."
                 raise ValueError(msg % (X.shape[1], self.theta_.shape[1]))
             # Put epsilon back in each time
-            self.sigma_[:, :] -= epsilon
+            self.sigma_[:, :] -= self.epsilon_
 
         classes = self.classes_
 
@@ -413,7 +417,7 @@ class GaussianNB(BaseNB):
             self.sigma_[i, :] = new_sigma
             self.class_count_[i] += N_i
 
-        self.sigma_[:, :] += epsilon
+        self.sigma_[:, :] += self.epsilon_
 
         # Update if only no priors is provided
         if self.priors is None:
@@ -461,16 +465,20 @@ class BaseDiscreteNB(BaseNB):
             self.class_log_prior_ = (np.log(self.class_count_) -
                                      np.log(self.class_count_.sum()))
         else:
-            self.class_log_prior_ = np.zeros(n_classes) - np.log(n_classes)
+            self.class_log_prior_ = np.full(n_classes, -np.log(n_classes))
 
     def _check_alpha(self):
-        if self.alpha < 0:
+        if np.min(self.alpha) < 0:
             raise ValueError('Smoothing parameter alpha = %.1e. '
-                             'alpha should be > 0.' % self.alpha)
-        if self.alpha < _ALPHA_MIN:
+                             'alpha should be > 0.' % np.min(self.alpha))
+        if isinstance(self.alpha, np.ndarray):
+            if not self.alpha.shape[0] == self.feature_count_.shape[1]:
+                raise ValueError("alpha should be a scalar or a numpy array "
+                                 "with shape [n_features]")
+        if np.min(self.alpha) < _ALPHA_MIN:
             warnings.warn('alpha too small will result in numeric errors, '
                           'setting alpha = %.1e' % _ALPHA_MIN)
-            return _ALPHA_MIN
+            return np.maximum(self.alpha, _ALPHA_MIN)
         return self.alpha
 
     def partial_fit(self, X, y, classes=None, sample_weight=None):
@@ -508,7 +516,6 @@ class BaseDiscreteNB(BaseNB):
         Returns
         -------
         self : object
-            Returns self.
         """
         X = check_array(X, accept_sparse='csr', dtype=np.float64)
         _, n_features = X.shape
@@ -574,7 +581,6 @@ class BaseDiscreteNB(BaseNB):
         Returns
         -------
         self : object
-            Returns self.
         """
         X, y = check_X_y(X, y, 'csr')
         _, n_features = X.shape
@@ -651,7 +657,7 @@ class MultinomialNB(BaseDiscreteNB):
     class_log_prior_ : array, shape (n_classes, )
         Smoothed empirical log probability for each class.
 
-    intercept_ : property
+    intercept_ : array, shape (n_classes, )
         Mirrors ``class_log_prior_`` for interpreting MultinomialNB
         as a linear model.
 
@@ -659,7 +665,7 @@ class MultinomialNB(BaseDiscreteNB):
         Empirical log probability of features
         given a class, ``P(x_i|y)``.
 
-    coef_ : property
+    coef_ : array, shape (n_classes, n_features)
         Mirrors ``feature_log_prob_`` for interpreting MultinomialNB
         as a linear model.
 
@@ -726,6 +732,110 @@ class MultinomialNB(BaseDiscreteNB):
                 self.class_log_prior_)
 
 
+class ComplementNB(BaseDiscreteNB):
+    """The Complement Naive Bayes classifier described in Rennie et al. (2003).
+
+    The Complement Naive Bayes classifier was designed to correct the "severe
+    assumptions" made by the standard Multinomial Naive Bayes classifier. It is
+    particularly suited for imbalanced data sets.
+
+    Read more in the :ref:`User Guide <complement_naive_bayes>`.
+
+    Parameters
+    ----------
+    alpha : float, optional (default=1.0)
+        Additive (Laplace/Lidstone) smoothing parameter (0 for no smoothing).
+
+    fit_prior : boolean, optional (default=True)
+        Only used in edge case with a single class in the training set.
+
+    class_prior : array-like, size (n_classes,), optional (default=None)
+        Prior probabilities of the classes. Not used.
+
+    norm : boolean, optional (default=False)
+        Whether or not a second normalization of the weights is performed. The
+        default behavior mirrors the implementations found in Mahout and Weka,
+        which do not follow the full algorithm described in Table 9 of the
+        paper.
+
+    Attributes
+    ----------
+    class_log_prior_ : array, shape (n_classes, )
+        Smoothed empirical log probability for each class. Only used in edge
+        case with a single class in the training set.
+
+    feature_log_prob_ : array, shape (n_classes, n_features)
+        Empirical weights for class complements.
+
+    class_count_ : array, shape (n_classes,)
+        Number of samples encountered for each class during fitting. This
+        value is weighted by the sample weight when provided.
+
+    feature_count_ : array, shape (n_classes, n_features)
+        Number of samples encountered for each (class, feature) during fitting.
+        This value is weighted by the sample weight when provided.
+
+    feature_all_ : array, shape (n_features,)
+        Number of samples encountered for each feature during fitting. This
+        value is weighted by the sample weight when provided.
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> X = np.random.randint(5, size=(6, 100))
+    >>> y = np.array([1, 2, 3, 4, 5, 6])
+    >>> from sklearn.naive_bayes import ComplementNB
+    >>> clf = ComplementNB()
+    >>> clf.fit(X, y)
+    ComplementNB(alpha=1.0, class_prior=None, fit_prior=True, norm=False)
+    >>> print(clf.predict(X[2:3]))
+    [3]
+
+    References
+    ----------
+    Rennie, J. D., Shih, L., Teevan, J., & Karger, D. R. (2003).
+    Tackling the poor assumptions of naive bayes text classifiers. In ICML
+    (Vol. 3, pp. 616-623).
+    https://people.csail.mit.edu/jrennie/papers/icml03-nb.pdf
+    """
+
+    def __init__(self, alpha=1.0, fit_prior=True, class_prior=None,
+                 norm=False):
+        self.alpha = alpha
+        self.fit_prior = fit_prior
+        self.class_prior = class_prior
+        self.norm = norm
+
+    def _count(self, X, Y):
+        """Count feature occurrences."""
+        if np.any((X.data if issparse(X) else X) < 0):
+            raise ValueError("Input X must be non-negative")
+        self.feature_count_ += safe_sparse_dot(Y.T, X)
+        self.class_count_ += Y.sum(axis=0)
+        self.feature_all_ = self.feature_count_.sum(axis=0)
+
+    def _update_feature_log_prob(self, alpha):
+        """Apply smoothing to raw counts and compute the weights."""
+        comp_count = self.feature_all_ + alpha - self.feature_count_
+        logged = np.log(comp_count / comp_count.sum(axis=1, keepdims=True))
+        # BaseNB.predict uses argmax, but ComplementNB operates with argmin.
+        feature_log_prob = -logged
+        if self.norm:
+            summed = logged.sum(axis=1, keepdims=True)
+            feature_log_prob = -feature_log_prob / summed
+        self.feature_log_prob_ = feature_log_prob
+
+    def _joint_log_likelihood(self, X):
+        """Calculate the class scores for the samples in X."""
+        check_is_fitted(self, "classes_")
+
+        X = check_array(X, accept_sparse="csr")
+        jll = safe_sparse_dot(X, self.feature_log_prob_.T)
+        if len(self.classes_) == 1:
+            jll += self.class_log_prior_
+        return jll
+
+
 class BernoulliNB(BaseDiscreteNB):
     """Naive Bayes classifier for multivariate Bernoulli models.
 
